package benten.cat.tm.engine.omegat.ui;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jface.preference.IPreferenceStore;
import org.omegat.util.StaticUtils;
import org.osgi.framework.BundleContext;

import benten.cat.tm.core.BentenTmDriver;
import benten.cat.tm.core.BentenTmDriverManager;
import benten.cat.tm.core.BentenTmEngine;
import benten.cat.tm.data.AbstractBentenTmDataTmxPlugin;
import benten.cat.tm.engine.AbstractBentenTmEnginePlugin;
import benten.cat.tm.engine.omegat.DefaultTmDriver;
import benten.cat.tm.engine.omegat.ui.messages.BentenTmPluginMessages;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;

/**
 * デフォルトの Benten TM (翻訳メモリ) エンジン・プラグイン。
 *
 * ★基本設計「共通機能: 翻訳メモリー・エンジン機能」に対応します。
 */
public class BentenTmPlugin extends AbstractBentenTmEnginePlugin {
	/**
	 * Benten デフォルト TM エンジン・プラグインのためのメッセージ。
	 */
	protected static final BentenTmPluginMessages fMsg = new BentenTmPluginMessages();

	private static final String DEFAULT_TMX = "default tmx"; //$NON-NLS-1$

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * TM エンジンのドライバーを取得します。
	 *
	 * @return TM エンジン・ドライバーのインスタンス。
	 */
	@Override
	public BentenTmDriver getDriver() {
		try {
			// 自らのドライバーを自己登録します。
			Class.forName("benten.cat.tm.engine.omegat.DefaultTmDriver"); //$NON-NLS-1$
		} catch (final ClassNotFoundException e) {
			log(fMsg.getGetDriverClassNotFoundException());
			log(e);
		}

		return BentenTmDriverManager.getDriver(DefaultTmDriver.DRIVER_NAME);
	}

	/**
	 * TM エンジンのインスタンスを取得します。
	 *
	 * @return TM エンジンのインスタンス。
	 */
	@Override
	public BentenTmEngine getEngine() {
		if (fEngine == null) {
			fEngine = getDriver().getEngineInstance();
		}

		return fEngine;
	}

	@Override
	public void load() {
		getDefault().log(fMsg.getLoadI001());

		getEngine().unload();

		final File dummyOmegatPrefs = new File(StaticUtils.getConfigDir() + "omegat.prefs"); //$NON-NLS-1$
		if (!dummyOmegatPrefs.exists()) {
			try {
				final PrintWriter writer = new PrintWriter(dummyOmegatPrefs);
				writer.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); //$NON-NLS-1$
				writer.close();
			} catch (final IOException e) {
				getDefault().log(e);
			}
		}
		final File tmxDir = getTmxDir();
		try {
			getEngine().loadTmx(tmxDir);
		} catch (final IOException e) {
			getDefault().log(e);
		}
	}

	@SuppressWarnings("static-access")
	private File getTmxDir() {
		File result = null;
		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.tm.data.tmx"); //$NON-NLS-1$

		if (IS_DEBUG) {
			System.out.println(point);
		}

		if (point == null)
			return result;

		final Map<String, AbstractBentenTmDataTmxPlugin> tmxMap = new HashMap<String, AbstractBentenTmDataTmxPlugin>();

		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name"); //$NON-NLS-1$
					Object tmxPluginClass;
					tmxPluginClass = cfgElem.createExecutableExtension("class"); //$NON-NLS-1$
					if (tmxPluginClass instanceof Plugin) {
						tmxMap.put(name, (AbstractBentenTmDataTmxPlugin) tmxPluginClass);
					}
				} catch (final CoreException e) {
					getDefault().log(e);
				}
			}

		}

		final AbstractBentenTmDataTmxPlugin tmxPlugin = tmxMap.get(DEFAULT_TMX);
		if (tmxPlugin == null)
			return result;

		final IPreferenceStore store = BentenPreference.getStore();
		final boolean useTmxLocation = store.getBoolean(Preference.USE_TMX_LOCATION.name());
		if (useTmxLocation) {
			final String tmxFolder = store.getString(Preference.TMX_LOCATION.name());
			result = new File(tmxFolder);
		} else {
			result = tmxPlugin.getDefault().getFile("/tmx"); //$NON-NLS-1$
		}

		getDefault().log(fMsg.getGetTmxDirI001(result.toString()));
		return result;
	}
}
